UNIT CustomTV;

TYPE TCustomTV = CLASS(TCustomTreeView)
     PRIVATE
       FDesigner : TFormDesigner;
       PROCEDURE CMDesignHitTest(VAR Message: TCMDesignHitTest); MESSAGE CM_DESIGNHITTEST;
       PROCEDURE CNNotify(VAR Message: TWMNotify); MESSAGE CN_NOTIFY;
       PROCEDURE WMLButtonDown(VAR Message: TWMLButtonDown); MESSAGE WM_LBUTTONDOWN;
       PROCEDURE WMRButtonDown(VAR Message: TWMRButtonDown); MESSAGE WM_RBUTTONDOWN;
       procedure WMNCHitTest(var Message: TWMNCHitTest); message WM_NCHITTEST;
     PROTECTED
       CONSTRUCTOR Create(AOwner: TComponent); OVERRIDE;
       PROCEDURE DragOver(Source: TObject; X, Y: Integer; State: TDragState; VAR Accept: Boolean); OVERRIDE;
       PROCEDURE DragDrop(Source: TObject; X, Y: Integer); OVERRIDE;
       PROCEDURE WndProc(VAR Message: TMessage); OVERRIDE;
     PUBLIC
       PROPERTY Designer: TFormDesigner READ FDesigner;
     END;

//------------------------------------------------------------------------------

IMPLEMENTATION

USES CustomDraw, Dialogs, Forms, SysUtils, Windows;

//------------------------------------------------------------------------------

PROCEDURE TCustomTV.CMDesignHitTest(VAR Message: TCMDesignHitTest);

BEGIN
  IF htOnItem IN GetHitTestInfoAt(Message.XPos,Message.YPos) THEN Message.Result:=1
                                                             ELSE Message.Result:=0;
END;

//------------------------------------------------------------------------------

PROCEDURE TCustomTV.CNNotify(VAR Message: TWMNotify);

// Catch the (reflected WM_NOTIFY) notification message, to handle custom drawing.

VAR TVcd : TNMTVCustomDraw;

BEGIN
  // Check, if it is the custom draw message (others will be handled by the
  // default handler).
  IF Message.NMHdr^.code = NM_CUSTOMDRAW THEN
  BEGIN
    // In this case we interpret the lParam (covered by NMHdr) as pointer to
    // a custom draw structur (as defined in 'CustomDraw.pas', which should be
    // in 'CommCtrl.pas', but Borland 'forgot' it).
    TVcd:=PNMTVCustomDraw(Pointer(Message.NMHdr))^;
    CASE TVcd.nmcd.dwDrawStage OF
      CDDS_PREPAINT     : Message.Result:=CDRF_NOTIFYITEMDRAW;
      CDDS_ITEMPREPAINT : IF Items.GetNode(HTReeItem(TVcd.nmcd.dwitemSpec)).Level = 1 THEN
                          BEGIN
                            SetTextColor(TVcd.nmcd.hdc,ColorToRGB(clTeal));
                            Message.Result:=CDRF_DODEFAULT;
                            //SelectObject(nmcd.hdc,TempFont.Handle);
                            //Message.Result:=CDRF_NEWFONT;
                          END
                          ELSE Message.Result:=CDRF_DODEFAULT
    ELSE Message.Result:=0;
    END;
  END
  ELSE INHERITED;
END;

//------------------------------------------------------------------------------

PROCEDURE TCustomTV.WMLButtonDown(VAR Message: TWMLButtonDown);

VAR Node : TTreeNode;

BEGIN
  INHERITED;
  Node:=GetNodeAt(Message.XPos,Message.YPos);
  IF assigned(Node) THEN
  BEGIN
    Node.Selected:=True;
    Node.Focused:=True;
    IF (csDesigning IN ComponentState) THEN Designer.SelectComponent(TComponent(Node.Data));
    IF DragMode = dmManual THEN BeginDrag(False);
  END
  ELSE
    IF assigned(Selected) THEN Selected.Selected:=False;
END;

//------------------------------------------------------------------------------

PROCEDURE TCustomTV.WMNCHitTest(VAR Message: TWMNCHitTest);

BEGIN
  DefaultHandler(Message);
END;

//------------------------------------------------------------------------------

PROCEDURE TCustomTV.DragOver(Source: TObject; X, Y: Integer; State: TDragState; VAR Accept: Boolean);

VAR SourceNode,
    DestinationNode : TTreeNode;

BEGIN
  INHERITED DragOver(Source,X,Y,State,Accept); // will call OnDragOver
  IF Source IS TCustomTreeView THEN
  BEGIN
    DestinationNode:=GetNodeAt(X,Y);
    SourceNode:=TCustomTV(Source).Selected;
    // formulate conditions for acceptance of a drop operation
    IF assigned(DestinationNode) AND ... THEN
    BEGIN
      Accept:=True;
      :
      :
    END;
  END;
END;

//------------------------------------------------------------------------------

PROCEDURE TCustomTV.DragDrop(Source: TObject; X, Y: Integer);

VAR SourceNode,
    DestinationNode   : TTreeNode;

BEGIN
  DestinationNode:=DropTarget;
  IF assigned(DestinationNode)   AND
     (Source IS TCustomTreeView) THEN
  BEGIN
    Items.BeginUpdate;
    SourceNode:=TCustomTreeView(Source).Selected;
    // do insert, delete, move here
    Items.EndUpdate;
  END;
  INHERITED DragDrop(Source,X,Y); // will call OnDragDrop
END;

//------------------------------------------------------------------------------

PROCEDURE TCustomTV.WndProc(var Message: TMessage);

// this is a very tricky way to get drag messages at design time

BEGIN
  IF Message.Msg = CM_DRAG THEN Dispatch(Message)
                           ELSE INHERITED WndProc(Message);
END;

//------------------------------------------------------------------------------

CONSTRUCTOR TCustomTV.Create(AOwner: TComponent);

BEGIN
  INHERITED Create(AOwner);
  // at design time we use the owners designer interface
  IF csDesigning IN ComponentState THEN FDesigner:=TForm(AOwner).Designer AS TFormDesigner;
END;

//------------------------------------------------------------------------------

END.
